#include "matrix.h"


error_code matrix_alloc(matrix *mat, int32_t l, int32_t c){
    mat->l = l;
    mat->c = c;
    if (l > 0 && c > 0){
        //On utilise la taille d'un pointer d'int32_t car on veut avoir assez de place pour stocker une ADRESSE, et non pas un int32_t !
        mat->data = malloc(l * sizeof(int32_t *));
        if (mat->data == NULL){
            return memory_error;
        }

        for (int i = 0; i < l; i++){
            mat->data[i] = malloc(c * sizeof(int32_t));
            if (mat->data[i] == NULL){
                return memory_error;
            }
        }
    }else{
        return out_of_bounds;
    }
   return ok; 
}


error_code matrix_init(matrix *mat, int32_t l, int32_t c,  int32_t val){
    mat->l = l;
    mat->c = c;
    if (l > 0 && c > 0){
        mat->data = malloc(l * sizeof(int32_t*));
        for (int i = 0; i < l; i++){
            mat->data[i] = malloc(c * sizeof(int32_t));
            for (int j = 0; j < c; j++){
                mat->data[i][j] = val;
            }
        }
        return ok;
    }else{
        return out_of_bounds;
    }
}


error_code matrix_destroy(matrix *mat){
    for (int i = 0; i < mat->l; i++){
        free(mat->data[i]);
        mat->data[i] = NULL;
    }
    free(mat->data);
    mat->data = NULL;

    mat->l = -1;
    mat->c = -1;
    return ok;
}


error_code matrix_init_from_array(matrix *mat, int32_t l, int32_t c, int32_t data[], int32_t s){
    if (s != l * c){
        return out_of_bounds;
    }

    error_code err = matrix_alloc(mat, l, c);  
    if (err != ok){
        return err;
    }
     
    for (int i = 0, a = 0; i < l; i++){
        for (int j = 0; j < c; j++, a++){
            mat->data[i][j] = data[a];
        }
    }
    return ok;
}


error_code matrix_clone(matrix *cloned, const matrix mat){
    if(mat.data == NULL){
        return uninitialized;
    }
        error_code err = matrix_alloc(cloned, mat.l, mat.c);  
        if (err != ok){
        return err;
    }
        for (int i = 0; i < cloned->l; i++){
            for (int j = 0; j < cloned->c; j++){
                cloned->data[i][j] = mat.data[i][j];
            } 
        }
    return ok;
}


error_code matrix_transpose(matrix *transposed,const matrix mat){
    if(mat.data == NULL){
        return uninitialized;
    }

   
    error_code err = matrix_alloc(transposed, mat.c, mat.l);  
    if (err != ok){
        return err;
    }

    for(int i = 0; i < transposed->l; i++){
        for (int j = 0; j < transposed->c; j++){
            transposed->data[i][j] = mat.data[j][i];
        }
    }
    return ok;
}


error_code matrix_print(const matrix mat){
    // TODO: handle errors !
    if(mat.data != NULL){
        if (mat.c > 0 && mat.l > 0){
            for (int i = 0; i < mat.l; i++){
                for (int y = 0; y < mat.c; y++){
                    printf("%d ", mat.data[i][y]);
                }
                printf("\n");
            }
            return ok;
        }else{
            return out_of_bounds;/*en réalité c'est plutôt une erreur de taille de matrice illégale*/
            }
    }else{
        return uninitialized;
    }
}


error_code matrix_extract_submatrix(matrix *sub,const matrix mat, int32_t l0, int32_t l1, int32_t c0, int32_t c1){
    if (mat.data == NULL){
        return uninitialized;
    }

    if ((l0 > l1) || (c0 > c1) || (l1 > mat.l) || (c1 > mat.c)) {
        return out_of_bounds;
    }

    int l = l1 - l0;
    int c = c1 - c0;

    printf("\n%d %d", l, c);

    error_code err = matrix_alloc(sub, l, c);
    if (err != ok){
        return err;
    }

    
    for(int i = l0, k = 0; i < l1; i++, k++){ 
        for(int j = c0, m = 0; j < c1; j++, m++){
            sub->data[k][m] = mat.data[i][j];
        }
    }
    return ok;
}


bool matrix_is_equal(matrix mat1, matrix mat2){
    if (mat1.c == mat2.c && mat1.l == mat2.l){
        for (int i = 0; i < mat1.l; i++){
            for (int j = 0; j < mat1.c; j++){
                if (mat1.data[i][j] != mat2.data[i][j]){
                    return false;
                }   
            }
        }
        return true;
    }else{
        return false;
    }
}


error_code matrix_get(int32_t *elem, const matrix mat, int32_t il, int32_t ic){
    if (mat.data == NULL){
        return uninitialized;
    }
    if (il >= mat.l || ic >= mat.c){
        return out_of_bounds;
    }

    for (int i = 0; i < mat.l; i++){
        for (int j = 0; j < mat.c; j++){
            if (i == il && j == ic){
                *elem = mat.data[i][j];
            }
        }
        
    }
    return ok;
}


error_code matrix_set(matrix mat, int32_t il, int32_t ic, int32_t elem){
    if (mat.data == NULL){
        return uninitialized;
    }
    if (il >= mat.l || ic >= mat.c){
        return out_of_bounds;
    }

    for (int i = 0; i < mat.l; i++){
        for (int j = 0; j < mat.c; j++){
            if (i == il && j == ic){
                mat.data[i][j] = elem;
            }
        }  
    }
    return ok;
}


error_code matrix_map_ip(matrix mat, void (*foo)(int *)){
    if (mat.data == NULL){
        return uninitialized;
    }

    for(int i = 0; i < mat.l; i++){
        for(int j = 0; j < mat.c; j++){
            foo(&mat.data[i][j]);
        }
    }
    return ok;
}


error_code matrix_map(matrix *mapped, const matrix mat, void (*foo)(int *)){
    if (mat.data == NULL){
        return uninitialized;
    }

    error_code err = matrix_alloc(mapped, mat.l, mat.c);
    if (err != ok){
        return err;
    }

    for(int i = 0; i < mat.l; i++){
        for(int j = 0; j < mat.c; j++){
            int value = mat.data[i][j];
            foo(&value);
            mapped->data[i][j] = value;
        }
    }
    return ok;
}


void print_error_name(uint8_t error){
    switch (error)
    {
    case 0:
        printf("'ok' \n");
        break;
    
    case 1:
        printf("'out of bounds' \n");
        break;
    
    case 2:
        printf("'memory error' \n");
        break;

    case 3:
        printf("'uninitialised'\n");
        break;

    }
}


void plus_one(uint32_t *value){
    *value += 1;
}