#ifndef MATRIX_H
#define MATRIX_H

#include <stdio.h>
#include <stdlib.h> 
#include <stdint.h>
#include <stdbool.h>


typedef struct matrix {
    int32_t c, l;
    int32_t ** data;
} matrix;

typedef enum error_code {
    ok, out_of_bounds, memory_error, uninitialized
} error_code;

error_code matrix_alloc(matrix *mat, int32_t l, int32_t c);
error_code matrix_init(matrix *mat, int32_t l, int32_t c,  int32_t val);
error_code matrix_destroy(matrix *mat);
error_code matrix_init_from_array(matrix *mat, int32_t l, int32_t c, int32_t data[], int32_t s);
error_code matrix_clone(matrix *cloned, const matrix mat);
error_code matrix_transpose(matrix* transposed,const matrix mat);
error_code matrix_print(const matrix mat);
error_code matrix_extract_submatrix(matrix *sub, const matrix mat, int32_t l0, int32_t l1, int32_t c0, int32_t c1);
bool matrix_is_equal(matrix mat1, matrix mat2);
error_code matrix_get(int32_t *elem, const matrix mat, int32_t il, int32_t ic);
error_code matrix_set(matrix mat, int32_t il, int32_t ic, int32_t elem);
error_code matrix_map_ip(matrix mat, void (*foo)(int *));
error_code matrix_map(matrix *mapped, const matrix mat, void (*foo)(int *));
void print_error_name(uint8_t error);
void plus_one(uint32_t *value);

#endif